% This program replicates Table 11 in "A Static Capital Buffer is 
% Hard to Beat" by Matthew Canzoneri, Behzad Diba, Luca Guerrieri 
% and Arsenii Mishin.
%
% This program simulates data under different simple rules for capital
% buffers -- change mycase below for each line in the table.

clear all

setpath_windows

global M_ oo_
global changes_names changes_vec

make_plots = 0;

nperiods = 1000;
maxiter = 20;

irfshock = char('eps_a','eps_isp');

% use this code to generate a sample
randn('seed',1);
nshocks = size(irfshock,1);
shockssequence = randn(nperiods,nshocks);

% select the static buffer for the desired portion of table 11 --- either
% 0.001 (i.e. 10 basis points)
% or
% 0.003 (i.e. 30 basis points)
gammap_increment_val = 0.003;

% toggle from 1 through 8 for each line in Table 11
% use call_regressions to verify the parameter choices hard-wired below
% (in call_regressions, remember to set tau_switch = 0 at the beginning of
% the program)
mycase = 1;

if mycase == 1
    % case 1 --- rule responds to investment prices
    changes_names = char('sigma_eps_a','sigma_eps_isp',...
        'rhop_a','rho_isp','phip_i','kappap','q_gammap','rho_gammap','gammap_increment');
    
    load my_estimates.mat distance_param;
    q_gammap = -0.087376143522716;
    rho_gammap = 0;
    changes_vec = [distance_param; q_gammap; rho_gammap; gammap_increment_val];
    
elseif mycase == 2
    % case 2 --- banking spread
    changes_names = char('sigma_eps_a','sigma_eps_isp',...
        'rhop_a','rho_isp','phip_i','kappap','banking_spread_gammap','rho_gammap','gammap_increment');
    
    
    load my_estimates.mat distance_param;
    banking_spread_gammap = 0.842025522804462;
    rho_gammap = 0;
    changes_vec = [distance_param; banking_spread_gammap; rho_gammap; gammap_increment_val];
    
elseif mycase == 3
    % case 3 --- banking spread
    changes_names = char('sigma_eps_a','sigma_eps_isp',...
        'rhop_a','rho_isp','phip_i','kappap','ytot_gammap','rho_gammap','gammap_increment');
    
    
    load my_estimates.mat distance_param;
    ytot_gammap = -0.0014360468817263;
    rho_gammap = 0;
    changes_vec = [distance_param; ytot_gammap; rho_gammap; gammap_increment_val];
    
    
elseif mycase == 4
    % case 4 --- rule responds to credit to GDP
    changes_names = char('sigma_eps_a','sigma_eps_isp',...
        'rhop_a','rho_isp','phip_i','kappap','ltot2ytot_gammap','rho_gammap','gammap_increment');
    
    
    load my_estimates.mat distance_param;
    ltot2ytot_gammap = -0.005092876144673;
    rho_gammap = 0;
    changes_vec = [distance_param; ltot2ytot_gammap; rho_gammap; gammap_increment_val];
    
elseif mycase == 5
    % case 5 --- rule resonds to credit to GDP, but flips the sign
    changes_names = char('sigma_eps_a','sigma_eps_isp',...
        'rhop_a','rho_isp','phip_i','kappap','ltot2ytot_gammap','rho_gammap','gammap_increment');
    
    
    load my_estimates.mat distance_param;
    ltot2ytot_gammap = 0.005092876144673;
    rho_gammap = 0;
    changes_vec = [distance_param; ltot2ytot_gammap; rho_gammap; gammap_increment_val];
    
    
elseif mycase == 6
    % case 6  --- expected safe returns and deposit rate
    changes_names = char('sigma_eps_a','sigma_eps_isp',...
        'rhop_a','rho_isp','phip_i','kappap','r_l_lead_gammap','r_d_gammap','rho_gammap','gammap_increment');
    
    load my_estimates.mat distance_param;
    r_l_lead_gammap = 5.945199809543128*100;
    r_d_gammap =  -5.945481343418757*100;
    
    rho_gammap = 0;
    changes_vec = [distance_param; r_l_lead_gammap; r_d_gammap; rho_gammap; gammap_increment_val];
    
    
elseif mycase == 7
    % case 7  --- all innovations, shocks, expected safe returns and deposit
    % rate
    changes_names = char('sigma_eps_a','sigma_eps_isp',...
        'rhop_a','rho_isp','phip_i','kappap',...
        'eps_a_gammap','a_gammap','eps_isp_gammap','isp_gammap',...
        'r_l_lead_gammap','r_d_gammap','rho_gammap','gammap_increment');
    
    load my_estimates.mat distance_param;
    eps_a_gammap = -0.000365069472084;
    a_gammap = 0.001613981568515;
    eps_isp_gammap =  0.000001288722283;
    isp_gammap = 0.099098494469654;
    r_l_lead_gammap = 0.941998518948214;
    r_d_gammap =  -1.044154907037963;
    rho_gammap = 0;
    
    changes_vec = [distance_param; eps_a_gammap; a_gammap; eps_isp_gammap; isp_gammap; r_l_lead_gammap; r_d_gammap; rho_gammap; gammap_increment_val];
    
elseif mycase == 8
    % case 8 --- All shock processes, innovations, and lagged capital requirement
    changes_names = char('sigma_eps_a','sigma_eps_isp',...
        'rhop_a','rho_isp','phip_i','kappap',...
        'eps_a_gammap','a_gammap','eps_isp_gammap','isp_gammap',...
        'rho_gammap','gammap_increment');
    
    load my_estimates.mat distance_param;
    eps_a_gammap = -0.000019662750011;
    a_gammap = 0.000744460328005;
    eps_isp_gammap =   0.000480899206620;
    isp_gammap =  -0.002238794098962;
    rho_gammap =  0.977315174827187;
    
    changes_vec = [distance_param; eps_a_gammap; a_gammap; eps_isp_gammap; isp_gammap;  rho_gammap; gammap_increment_val];
    
end



% model with capital requirements set by a simple rule
mod00 = 'model_safe_all_shocks';
mod11 = 'model_mixed_all_shocks';
mod10 = 'model_risky_all_shocks';
mod01 = 'model_spare_all_shocks';

% model with ramsey capital requirements
modnam2 = 'model_calibration';




constraint1 = 'chi2_upper_bar+chi2_upper_bar_ss<0';    % If the Lagrange multiplier on the
% no-shorting constraints for risky
% loans is slack, it means that I want
% to make risky loans.
% Accordingly, go into 11 or 10

constraint_relax1 = '((l+l_ss)<0)|(l_upper_bar<0)'; % If it turns out that the quantity of
% risky loans is negative,  00
% notice that l is set to 0 in the 10
% file, therefore the first condition
% is irrelevant when transition back
% from 10, but it might be relevant
% when transition back from 11.


constraint2 = '((chi2_lower_bar<0)&(l_upper_bar>0))'; % If the Lagrange multiplier on the
% no-shorting constraints for risky
% loans  and on safe loans are both slack, it means that I want
% to make risky and safe loans. Go to
% 01 or 11, but since if this
% condition is met constraint1 will
% also be active, we can only go into
% 11. Hence, 01 is spare.

constraint_relax2 ='((l+l_ss)<0)|(l_upper_bar<0)';  % return to 10 or 00, but
% notice that
% constraint_relax1 is identical,
% therefore, if either
% condition is violated
% we will go back to 00


% end of inputs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


[zdatalinear, zdatapiecewise, zdatass, oobase_, Mbase_, violvecbool ] = ...
    solve_two_constraints_add_violvecbool(mod00,mod10,mod01,mod11,...
    constraint1, constraint2,...
    constraint_relax1, constraint_relax2,...
    shockssequence,irfshock,nperiods,0,maxiter);

% repeat the simulation for the model with endogenous capital requirements

% set the static buffer to 0 when using the optimal rule
gammap_increment = 0;
changes_vec(end) = gammap_increment;

[zdatalinear2, zdatass2, oobase2_, Mbase2_ ] = ...
    solve_no_constraint(modnam2,...
    shockssequence,irfshock,nperiods);


prefix1 = 's1_';
prefix2 = 's2_';
for this_var = 1:Mbase_.endo_nbr
    eval([prefix1,deblank(Mbase_.endo_names(this_var,:)),'_irf=zdatapiecewise(:,this_var);'])
    eval([prefix1,deblank(Mbase_.endo_names(this_var,:)),'_ss=zdatass(this_var);'])
    
    eval([prefix2,deblank(Mbase2_.endo_names(this_var,:)),'_irf=zdatalinear2(:,this_var);'])
    eval([prefix2,deblank(Mbase2_.endo_names(this_var,:)),'_ss=zdatass2(this_var);'])
    
end


%% stats


n_risk = length(find((s1_sigma_ss+s1_sigma_irf)>0.01))/(nperiods/4)*100;
display(['Average number of quarters with excessive risk-taking (per 100 years) ',num2str(n_risk,5)])

display(['Average deposit under Simple rule ',num2str(mean(s1_d_irf+s1_d_ss))])
display(['Average deposit under Ramsey rule ',num2str(mean(s2_d_irf+s2_d_ss))])


%% plots

if make_plots
    
    titlelist = char('Total output', 'Bank capital requirement',...
        'Consumption', 'Credit/GDP ratio',...
        'Expected Equity Return Spread (risky-safe)',  'Total capital',...
        'Expected Safe Equity Return','Investment Price' );
    
    
    line1 = 100*[ s1_ytot_irf/s1_ytot_ss, s1_gamma_var_irf,...
        s1_c_irf/s1_c_ss, s1_ltot2ytot_irf/4,...
        [s1_r_e_upper_bar_irf(2:end)+s1_r_e_upper_bar_ss-s1_r_e_irf(2:end)-s1_r_e_ss; nan], s1_ktot_irf/s1_ktot_ss,...
        [s1_r_e_irf(2:end); nan],s1_q_irf ];
    
    line2 = 100*[ s2_ytot_irf/s2_ytot_ss, s2_gamma_var_irf,...
        s2_c_irf/s2_c_ss, s2_ltot2ytot_irf/4,...
        [s2_r_e_upper_bar_irf(2:end)-s2_r_e_irf(2:end); nan], s2_ktot_irf/s2_ktot_ss,...
        [s2_r_e_irf(2:end); nan],s2_q_irf ];
    
    %1-normcdf((s1_epsilon_star_irf(1:30)+s1_epsilon_star_ss+xip)/taup)
    %(s1_k_r_irf+s1_k_r_ss)./(s1_k_r_irf+s1_k_r_ss+s1_k_irf+s1_k_ss)/100];
    
    percent = 'Percent';
    ppt = 'Perc. Point';
    level = 'Level';
    ylabels = char(percent,ppt,...
        percent,ppt,...
        ppt,percent,...
        ppt, percent);
    figlabel = '';
    
    
    
    legendlist  = char('Endog. Capital Requirements','Fixed Capital Requirements');
    makechart(titlelist,legendlist,figlabel,ylabels,line2,line1);
    
    
    reorder = [1 3 2 5 6 4];
    makechart_slide(titlelist(reorder,:),legendlist,figlabel,ylabels(reorder,:),line2(:,reorder),line1(:,reorder));
end




