function [welf_opt, welf_vec, ds_vec, ktot_vec, ltot_vec, c_vec, welfc_vec, zdatass_, oobase_, Mbase_, violvecguess ] = ...
    solve_gamma0_check(modnam_,modnamstar_,...
    shockssequence_,irfshock_,nperiods_,No_of_Bits_,max_switches_,init_)



global M_ oo_

errlist_ = [];

% solve the reference model linearly
eval(['dynare ',modnam_,' noclearall nolog '])
oobase_ = oo_;
Mbase_ = M_;

% import locally the values of parameters assigned in the reference .mod
% file
for i_indx_ = 1:Mbase_.param_nbr
  eval([Mbase_.param_names(i_indx_,:),'= M_.params(i_indx_);']);
end

% Create steady state values of the variables if needed for processing the constraint
for i=1:Mbase_.endo_nbr
   eval([deblank(Mbase_.endo_names(i,:)) '_ss = oobase_.dr.ys(i); ']);
end


% parse the .mod file for the alternative regime
eval(['dynare ',modnamstar_,' noclearall nolog '])
oostar_ = oo_;
Mstar_ = M_;


% check inputs
if ~strcmp(Mbase_.endo_names,Mstar_.endo_names)
    error('The two .mod files need to have exactly the same endogenous variables declared in the same order')
end

if ~strcmp(Mbase_.exo_names,Mstar_.exo_names)
    error('The two .mod files need to have exactly the same exogenous variables declared in the same order')
end

if ~strcmp(Mbase_.param_names,Mstar_.param_names)
    warning('The parameter list does not match across .mod files')
end

% ensure that the two models have the same parameters
% use the parameters for the base model.
Mstar_.params = Mbase_.params;

nvars_ = Mbase_.endo_nbr;
zdatass_ = oobase_.dr.ys;


% get the matrices holding the first derivatives for the model
% each regime is treated separately
[hm1_,h_,hl1_,Jbarmat_] = get_deriv(Mbase_,zdatass_);
cof_ = [hm1_,h_,hl1_];

[hm1_,h_,hl1_,Jstarbarmat_,resid_] = get_deriv(Mstar_,zdatass_);
cofstar_ = [hm1_,h_,hl1_];
Dstartbarmat_ = resid_;

if isfield(Mbase_,'nfwrd')
    % the latest Dynare distributions have moved nstatic and nfwrd
    [decrulea_,decruleb_]=get_pq(oobase_.dr,Mbase_.nstatic,Mbase_.nfwrd);
else
    [decrulea_,decruleb_]=get_pq(oobase_.dr,oobase_.dr.nstatic,oobase_.dr.nfwrd);
end

endog_ = M_.endo_names;
exog_ =  M_.exo_names;


% processes the constraints specified in the call to this function
% uppend a suffix to each endogenous variable

nshocks_ = size(shockssequence_,1);

% if necessary, set default values for optional arguments
if ~exist('init_')
    init_ = zeros(nvars_,1);
end

if ~exist('maxiter_')
    maxiter_ = 20;
end

if ~exist('nperiods_')
    nperiods_ = 100;
end


% set some initial conditions and loop through the shocks 
% period by period
init_orig_ = init_;
zdatapiecewise_ = zeros(nperiods_,nvars_);
wishlist_ = endog_;
nwishes_ = size(wishlist_,1);


[violvecguess, N]= make_guesses(No_of_Bits_,nperiods_,max_switches_);
violvecbool_ = zeros(nperiods_+1,1);

ishock_ = 1;

nguesses = size(violvecguess,1);


welf_pos = strmatch('welf',Mbase_.endo_names,'exact');

ds_pos = strmatch('ds',Mbase_.endo_names,'exact');

ktot_pos = strmatch('ktot',Mbase_.endo_names,'exact');

ltot_pos = strmatch('ltot',Mbase_.endo_names,'exact');

c_pos = strmatch('c',Mbase_.endo_names,'exact');

welfc_pos = strmatch('welfc',Mbase_.endo_names,'exact');


welf_vec = zeros(nguesses,1);

ds_vec = zeros(nguesses,nperiods_+1);

ktot_vec = zeros(nguesses,nperiods_+1);

ltot_vec = zeros(nguesses,nperiods_+1);

c_vec = zeros(nguesses,nperiods_+1);

welfc_vec = zeros(nguesses,1);

for this_guess = 1:nguesses
    
    violvecbool_ = violvecguess(this_guess,:)';
   
    
    
        
        % analyze when each regime starts based on current guess
        [regime regimestart]=map_regime(violvecbool_);
        
        
        
        % get the hypothesized piece wise linear solution
        [zdatalinear_]=mkdatap_anticipated(nperiods_,decrulea_,decruleb_,...
            cof_,Jbarmat_,cofstar_,Jstarbarmat_,Dstartbarmat_,...
            regime,regimestart,violvecbool_,...
            endog_,exog_,irfshock_,shockssequence_(ishock_,:),init_);
        
        welf_vec(this_guess) = zdatalinear_(1,welf_pos)+zdatass_(welf_pos);
        
        ds_vec(this_guess,:) = zdatalinear_(:,ds_pos)';%+zdatass_(ds_pos);
        
        ktot_vec(this_guess,:) = zdatalinear_(:,ktot_pos)';%+zdatass_(ktot_pos);
        
        ltot_vec(this_guess,:) = zdatalinear_(:,ltot_pos)';%+zdatass_(ltot_pos);
        
        c_vec(this_guess,:) = zdatalinear_(:,c_pos)';%+zdatass_(c_pos);
        
        welfc_vec(this_guess) = zdatalinear_(1,welfc_pos)+zdatass_(welfc_pos);        
end


zdatalinear_ = mkdata(max(nperiods_,size(shockssequence_,1)),...
                  decrulea_,decruleb_,endog_,exog_,...
                  wishlist_,irfshock_,shockssequence_,init_orig_);
              
welf_opt = zdatalinear_(1,welf_pos)+zdatass_(welf_pos);
    


