% The function solves the model (obtains piece-wise linear solution) under 
% the initial guess which comes from finding the first instance in which the
% banks would be better off switching for one period only

function [zdatalinear_, zdatapiecewise_, zdatass_, oo00_, M00_, violvecbool_ ] = ...
    solve_two_constraints_check2(modnam_00_,modnam_10_,modnam_01_,modnam_11_,...
    shockssequence_,irfshock_,nperiods_,maxiter_,init_)


global M_ oo_



% solve model
eval(['dynare ',modnam_00_,' noclearall nolog'])
oo00_ = oo_;
M00_ = M_;


for i=1:M00_.endo_nbr
    eval([deblank(M00_.endo_names(i,:)) '_ss = oo00_.dr.ys(i); ']);
end

for i_indx_ = 1:M00_.param_nbr
    eval([M00_.param_names(i_indx_,:),'= M00_.params(i_indx_);']);
end



eval(['dynare ',modnam_10_,' noclearall'])
oo10_ = oo_;
M10_ = M_;

eval(['dynare ',modnam_01_,' noclearall'])
oo01_ = oo_;
M01_ = M_;

eval(['dynare ',modnam_11_,' noclearall'])
oo11_ = oo_;
M11_ = M_;


% do some error checking

% check inputs
if ~strcmp(M00_.endo_names,M10_.endo_names)
    error([modnam_00_,' and ',modnam_10_,' need to have exactly the same endogenous variables and they need to be declared in the same order'])
end

if ~strcmp(M00_.exo_names,M10_.exo_names)
    error([modnam_00_,' and ',modnam_10_,' need to have exactly the same exogenous variables and they need to be declared in the same order'])
end

if ~strcmp(M00_.param_names,M10_.param_names)
    warning(['The parameter list does not match across the files ',modnam_00_,' and ',modnam_10_])
end


if ~strcmp(M00_.endo_names,M01_.endo_names)
    error([modnam_00,' and ',modnam_01_,' need to have exactly the same endogenous variables and they need to be declared in the same order'])
end

if ~strcmp(M00_.exo_names,M01_.exo_names)
    error([modnam_00_,' and ',modnam_01_,' need to have exactly the same exogenous variables and they need to be declared in the same order'])
end

if ~strcmp(M00_.param_names,M01_.param_names)
    warning(['The parameter list does not match across the files ',modnam_00_,' and ',modnam_01_])
end


if ~strcmp(M00_.endo_names,M11_.endo_names)
    error([modnam_00_,' and ',modnam_11_,' need to have exactly the same endogenous variables and they need to be declared in the same order'])
end

if ~strcmp(M00_.exo_names,M11_.exo_names)
    error([modnam_00_,' and ',modnam_11_,' need to have exactly the same exogenous variables and they need to be declared in the same order'])
end

if ~strcmp(M00_.param_names,M11_.param_names)
    warning(['The parameter list does not match across the files ',modnam_00_,' and ',modnam_11_])
end





nvars_ = M00_.endo_nbr;
zdatass_ = oo00_.dr.ys;


[hm1_,h_,hl1_,Jbarmat_] = get_deriv(M00_,zdatass_);
cof_ = [hm1_,h_,hl1_];


M10_.params = M00_.params;
[hm1_,h_,hl1_,Jbarmat10_,resid_] = get_deriv(M10_,zdatass_);
cof10_ = [hm1_,h_,hl1_];
Dbarmat10_ = resid_;

M01_.params = M00_.params;
[hm1_,h_,hl1_,Jbarmat01_,resid_] = get_deriv(M01_,zdatass_);
cof01_ = [hm1_,h_,hl1_];
Dbarmat01_ = resid_;

M11_.params = M00_.params;
[hm1_,h_,hl1_,Jbarmat11_,resid_] = get_deriv(M11_,zdatass_);
cof11_ = [hm1_,h_,hl1_];
Dbarmat11_ = resid_;


if isfield(M00_,'nfwrd')  % needed for bakward compatibility with older Dynare releases
    [decrulea,decruleb]=get_pq(oo00_.dr,M00_.nstatic,M00_.nfwrd);
else
    [decrulea,decruleb]=get_pq(oo00_.dr,oo00_.dr.nstatic,oo00_.dr.nfwrd);
end
endog_ = M00_.endo_names;
exog_ =  M00_.exo_names;


if ~exist('init_','var')
    init_ = zeros(nvars_,1);
end
init_orig = init_;

if ~exist('maxiter_','var')
    maxiter_ = 20;
end





% set some initial conditions and loop through the shocks
% period by period
criterion_pos=strmatch('expected_profits',M00_.endo_names,'exact');
violvecbool_ = zeros(nperiods_+1,1);
nshocks = size(shockssequence_,1);
switch_ = 0;
zdatapiecewise_ = zeros(nperiods_,nvars_);

for ishock_ = 1:nshocks
    
    if max(violvecbool_) == 0 % check for initial guess of regime switches otherwise
        % use the violvecbool_ from the previous shock in the sequence
        
        violvecbool_mat  = [[0; violvecbool_(1:end-1)] violvecbool_];
        [regime1, regimestart1]=map_regime(violvecbool_mat(:,1));
        [regime2, regimestart2]=map_regime(violvecbool_mat(:,2));
        
        
        [zdatalinear_]=mkdatap_anticipated_2constraints(nperiods_,decrulea,decruleb,...
            cof_,Jbarmat_,...
            cof10_,Jbarmat10_,Dbarmat10_,...
            cof01_,Jbarmat01_,Dbarmat01_,...
            cof11_,Jbarmat11_,Dbarmat11_,...
            regime1,regimestart1,...
            regime2,regimestart2,...
            violvecbool_mat,endog_,exog_,...
            irfshock_,shockssequence_(ishock_,:),init_);
        
        criterion=zdatalinear_(:,criterion_pos);
        
        
        this_period = 1;
        
        while this_period<nperiods_ && ~switch_
            
            violvecbool_= zeros(nperiods_+1,1);
            violvecbool_(this_period)=1;
            
            violvecbool_mat = [ [0;violvecbool_(1:end-1)] violvecbool_ ];
            [regime_1, regimestart_1]=extended_map_regime(violvecbool_mat(:,1));
            [regime_2, regimestart_2]=extended_map_regime(violvecbool_mat(:,2));
            
            %     [zdata_check]=mkdatap_anticipated(nperiods_,decrulea_,decruleb_,...
            %         cof_,Jbarmat_,cofstar_,Jstarbarmat_,Dstartbarmat_,...
            %         regime_check,regimestart_check,violvecbool_check,...
            %         endog_,exog_,irfshock_,shockssequence_,init_);
            
            
            [zdata_check]=mkdatap_anticipated_2constraints(nperiods_,decrulea,decruleb,...
                cof_,Jbarmat_,...
                cof10_,Jbarmat10_,Dbarmat10_,...
                cof01_,Jbarmat01_,Dbarmat01_,...
                cof11_,Jbarmat11_,Dbarmat11_,...
                regime_1,regimestart_1,...
                regime_2,regimestart_2,...
                violvecbool_mat,endog_,exog_,...
                irfshock_,shockssequence_(ishock_,:),init_);
            if zdata_check(this_period,criterion_pos)>criterion(this_period)
                
                switch_=1;
                
                violvecbool_(this_period) =1;
                [regime, regimestart]=extended_map_regime(violvecbool_);
                current_guess = zdata_check;
                
            end
            this_period = this_period+1;
            
        end
        %plot(violvecbool_)
        %pause
        % construct new guess by pooling all the periods in which I was better off
        % switching for one period only
        
        
        % % figure
        % % plot(violvecbool_); pause
        
        %initialize variables for loop below
    end
    
    
    converged_switch = 0;
    this_iter  = 1;
    
    if switch_ == 0
        violvecbool_ = 0*violvecbool_;
        init_ = zdatalinear_(1,:);
        zdatapiecewise_(ishock_,:)=init_;
        init_= init_';
    else
        % check whether I can expand the periods when we switch (or contract)
        
        
        
        while this_iter<=maxiter_ && ~converged_switch
            
            [contract_right_switch, regime, regimestart, violvecbool_, new_guess] = contract_right_two_constraints(criterion_pos,nperiods_,decrulea,decruleb,...
                cof_,Jbarmat_,...
                cof10_,Jbarmat10_,Dbarmat10_,...
                cof01_,Jbarmat01_,Dbarmat01_,...
                cof11_,Jbarmat11_,Dbarmat11_,...
                regime,regimestart,violvecbool_,...
                endog_,exog_,...
                irfshock_,shockssequence_(ishock_,:),init_, current_guess);
            
            
            if contract_right_switch
                current_guess = new_guess;
            end
            
            if ~contract_right_switch
                [expand_right_switch, regime, regimestart, violvecbool_,new_guess]=expand_right_two_constraints(criterion_pos,nperiods_,decrulea,decruleb,...
                    cof_,Jbarmat_,...
                    cof10_,Jbarmat10_,Dbarmat10_,...
                    cof01_,Jbarmat01_,Dbarmat01_,...
                    cof11_,Jbarmat11_,Dbarmat11_,...
                    regime,regimestart,violvecbool_,...
                    endog_,exog_,...
                    irfshock_,shockssequence_(ishock_,:),init_, current_guess);
                if expand_right_switch
                    current_guess = new_guess;
                end
            end
            
            %plot(violvecbool_); pause
            
            
            [contract_left_switch, regime, regimestart, violvecbool_,new_guess]=contract_left_two_constraints(criterion_pos,nperiods_,decrulea,decruleb,...
                cof_,Jbarmat_,...
                cof10_,Jbarmat10_,Dbarmat10_,...
                cof01_,Jbarmat01_,Dbarmat01_,...
                cof11_,Jbarmat11_,Dbarmat11_,...
                regime,regimestart,violvecbool_,...
                endog_,exog_,...
                irfshock_,shockssequence_(ishock_,:),init_, current_guess);
            if contract_left_switch
                current_guess = new_guess;
            end
            
            
            if ~contract_left_switch
                
                [expand_left_switch, regime, regimestart, violvecbool_,new_guess]=expand_left_two_constraints(criterion_pos,nperiods_,decrulea,decruleb,...
                    cof_,Jbarmat_,...
                    cof10_,Jbarmat10_,Dbarmat10_,...
                    cof01_,Jbarmat01_,Dbarmat01_,...
                    cof11_,Jbarmat11_,Dbarmat11_,...
                    regime,regimestart,violvecbool_,...
                    endog_,exog_,...
                    irfshock_,shockssequence_(ishock_,:),init_, current_guess);
                if expand_left_switch
                    current_guess = new_guess;
                end
            end
            
            if ~contract_right_switch && ~contract_left_switch && ~expand_right_switch && ~expand_left_switch
                converged_switch = 1;
            else
                contract_right_switch = 0;
                contract_left_switch = 0;
                expand_right_switch = 0;
                expand_left_switch = 0;
            end
            
            %plot(violvecbool_); pause
            
            
        end
        
        if ~converged_switch
           error('Unable to converge -- consider increase maxiter') 
        end
        
        init_ = current_guess(1,:);
        zdatapiecewise_(ishock_,:)=init_;
        init_= init_';
        
        if ishock_ < nshocks
        
        % update guess
        violvecbool_ = [violvecbool_(2:end); 0];
        violvecbool_mat = [ [0;violvecbool_(1:end-1)] violvecbool_ ];
            [regime_1, regimestart_1]=extended_map_regime(violvecbool_mat(:,1));
            [regime_2, regimestart_2]=extended_map_regime(violvecbool_mat(:,2));
            
        
        [current_guess]=mkdatap_anticipated_2constraints(nperiods_,decrulea,decruleb,...
                cof_,Jbarmat_,...
                cof10_,Jbarmat10_,Dbarmat10_,...
                cof01_,Jbarmat01_,Dbarmat01_,...
                cof11_,Jbarmat11_,Dbarmat11_,...
                regime_1,regimestart_1,...
                regime_2,regimestart_2,...
                violvecbool_mat,endog_,exog_,...
                irfshock_,shockssequence_(ishock_+1,:),init_);
        end
        %zdatapiecewise_= current_guess;  % verify what to do with multiple shocks
    end
    
end

if switch_==0
    zdatapiecewise_(ishock_+1:end,:)=zdatalinear_(2:nperiods_-ishock_+1,:);
    zdatalinear_=zdatapiecewise_;
else
    zdatapiecewise_(ishock_+1:end,:)=current_guess(2:nperiods_-ishock_+1,:);
    zdatalinear_ = mkdata(nperiods_,decrulea,decruleb,endog_,exog_,endog_,irfshock_,shockssequence_,init_orig);
end

